@@ -23,33 +23,38 @@
 **
 ****************************************************************************/
 
-#include "algorithm.h"
 #include "filesearch.h"
+
+#include "algorithm.h"
 #include "fileutils.h"
 #include "mapreduce.h"
+#include "qtcassert.h"
+#include "stringutils.h"
 
 #include <QCoreApplication>
+#include <QLoggingCategory>
 #include <QMutex>
-#include <QRegExp>
 #include <QRegularExpression>
 #include <QTextCodec>
 
 #include <cctype>
 
+Q_LOGGING_CATEGORY(log, "qtc.utils.filesearch", QtWarningMsg)
+
 using namespace Utils;
 
 static inline QString msgCanceled(const QString &searchTerm, int numMatches, int numFilesSearched)
 {
     return QCoreApplication::translate("Utils::FileSearch",
                                        "%1: canceled. %n occurrences found in %2 files.",
-                                       0, numMatches).arg(searchTerm).arg(numFilesSearched);
+                                       nullptr, numMatches).arg(searchTerm).arg(numFilesSearched);
 }
 
 static inline QString msgFound(const QString &searchTerm, int numMatches, int numFilesSearched)
 {
     return QCoreApplication::translate("Utils::FileSearch",
                                        "%1: %n occurrences found in %2 files.",
-                                       0, numMatches).arg(searchTerm).arg(numFilesSearched);
+                                       nullptr, numMatches).arg(searchTerm).arg(numFilesSearched);
 }
 
 namespace {
@@ -64,19 +69,20 @@ QString clippedText(const QString &text, int maxLength)
 }
 
 // returns success
-bool openStream(const QString &filePath, QTextCodec *encoding, QTextStream *stream, QFile *file,
-                QString *tempString,
-                const QMap<QString, QString> &fileToContentsMap)
+bool getFileContent(const QString &filePath,
+                    QTextCodec *encoding,
+                    QString *tempString,
+                    const QMap<QString, QString> &fileToContentsMap)
 {
     if (fileToContentsMap.contains(filePath)) {
         *tempString = fileToContentsMap.value(filePath);
-        stream->setString(tempString);
     } else {
-        file->setFileName(filePath);
-        if (!file->open(QIODevice::ReadOnly))
+        QFile file(filePath);
+        if (!file.open(QIODevice::ReadOnly))
             return false;
-        stream->setDevice(file);
-        stream->setCodec(encoding);
+        const QByteArray content = file.readAll();
+        *tempString = QTC_GUARD(encoding) ? encoding->toUnicode(content)
+                                          : QTextCodec::codecForLocale()->toUnicode(content);
     }
     return true;
 }
@@ -137,16 +143,17 @@ void FileSearch::operator()(QFutureInterface<FileSearchResultList> &futureInterf
 {
     if (futureInterface.isCanceled())
         return;
+    qCDebug(log) << "Searching in" << item.filePath;
     futureInterface.setProgressRange(0, 1);
     futureInterface.setProgressValue(0);
     FileSearchResultList results;
-    QFile file;
-    QTextStream stream;
     QString tempString;
-    if (!openStream(item.filePath, item.encoding, &stream, &file, &tempString, fileToContentsMap)) {
+    if (!getFileContent(item.filePath, item.encoding, &tempString, fileToContentsMap)) {
+        qCDebug(log) << "- failed to get content for" << item.filePath;
         futureInterface.cancel(); // failure
         return;
     }
+    QTextStream stream(&tempString);
     int lineNr = 0;
 
     while (!stream.atEnd()) {
@@ -216,12 +223,11 @@ void FileSearch::operator()(QFutureInterface<FileSearchResultList> &futureInterf
         if (futureInterface.isCanceled())
             break;
     }
-    if (file.isOpen())
-        file.close();
     if (!futureInterface.isCanceled()) {
         futureInterface.reportResult(results);
         futureInterface.setProgressValue(1);
     }
+    qCDebug(log) << "- finished searching in" << item.filePath;
 }
 
 FileSearchRegExp::FileSearchRegExp(const QString &searchTerm, QTextDocument::FindFlags flags,
@@ -251,18 +257,23 @@ QRegularExpressionMatch FileSearchRegExp::doGuardedMatch(const QString &line, in
 void FileSearchRegExp::operator()(QFutureInterface<FileSearchResultList> &futureInterface,
                                   const FileIterator::Item &item) const
 {
+    if (!expression.isValid()) {
+        futureInterface.cancel();
+        return;
+    }
     if (futureInterface.isCanceled())
         return;
+    qCDebug(log) << "Searching in" << item.filePath;
     futureInterface.setProgressRange(0, 1);
     futureInterface.setProgressValue(0);
     FileSearchResultList results;
-    QFile file;
-    QTextStream stream;
     QString tempString;
-    if (!openStream(item.filePath, item.encoding, &stream, &file, &tempString, fileToContentsMap)) {
+    if (!getFileContent(item.filePath, item.encoding, &tempString, fileToContentsMap)) {
+        qCDebug(log) << "- failed to get content for" << item.filePath;
         futureInterface.cancel(); // failure
         return;
     }
+    QTextStream stream(&tempString);
     int lineNr = 0;
 
     QString line;
@@ -289,19 +300,18 @@ void FileSearchRegExp::operator()(QFutureInterface<FileSearchResultList> &future
         if (futureInterface.isCanceled())
             break;
     }
-    if (file.isOpen())
-        file.close();
     if (!futureInterface.isCanceled()) {
         futureInterface.reportResult(results);
         futureInterface.setProgressValue(1);
     }
+    qCDebug(log) << "- finished searching in" << item.filePath;
 }
 
 struct SearchState
 {
     SearchState(const QString &term, FileIterator *iterator) : searchTerm(term), files(iterator) {}
     QString searchTerm;
-    FileIterator *files = 0;
+    FileIterator *files = nullptr;
     FileSearchResultList cachedResults;
     int numFilesSearched = 0;
     int numMatches = 0;
@@ -385,12 +395,13 @@ QFuture<FileSearchResultList> Utils::findInFilesRegExp(const QString &searchTerm
 
 QString Utils::expandRegExpReplacement(const QString &replaceText, const QStringList &capturedTexts)
 {
-    // handles \1 \\ \& & \t \n
+    // handles \1 \\ \& \t \n $1 $$ $&
     QString result;
     const int numCaptures = capturedTexts.size() - 1;
-    for (int i = 0; i < replaceText.length(); ++i) {
+    const int replaceLength = replaceText.length();
+    for (int i = 0; i < replaceLength; ++i) {
         QChar c = replaceText.at(i);
-        if (c == QLatin1Char('\\') && i < replaceText.length() - 1) {
+        if (c == QLatin1Char('\\') && i < replaceLength - 1) {
             c = replaceText.at(++i);
             if (c == QLatin1Char('\\')) {
                 result += QLatin1Char('\\');
@@ -404,16 +415,26 @@ QString Utils::expandRegExpReplacement(const QString &replaceText, const QString
                 int index = c.unicode()-'1';
                 if (index < numCaptures) {
                     result += capturedTexts.at(index+1);
-                } else {
-                    result += QLatin1Char('\\');
-                    result += c;
-                }
+                } // else add nothing
             } else {
                 result += QLatin1Char('\\');
                 result += c;
             }
-        } else if (c == QLatin1Char('&')) {
-            result += capturedTexts.at(0);
+        } else if (c== '$' && i < replaceLength - 1) {
+            c = replaceText.at(++i);
+            if (c == '$') {
+                result += '$';
+            } else if (c == '&') {
+                result += capturedTexts.at(0);
+            } else if (c.isDigit()) {
+                int index = c.unicode()-'1';
+                if (index < numCaptures) {
+                    result += capturedTexts.at(index+1);
+                } // else add nothing
+            } else {
+                result += '$';
+                result += c;
+            }
         } else {
             result += c;
         }
@@ -460,22 +481,24 @@ QString matchCaseReplacement(const QString &originalText, const QString &replace
 }
 } // namespace
 
-static QList<QRegExp> filtersToRegExps(const QStringList &filters)
+static QList<QRegularExpression> filtersToRegExps(const QStringList &filters)
 {
     return Utils::transform(filters, [](const QString &filter) {
-        return QRegExp(filter, Qt::CaseInsensitive, QRegExp::Wildcard);
+        return QRegularExpression(Utils::wildcardToRegularExpression(filter),
+                                  QRegularExpression::CaseInsensitiveOption);
     });
 }
 
-static bool matches(const QList<QRegExp> &exprList, const QString &filePath)
+static bool matches(const QList<QRegularExpression> &exprList, const QString &filePath)
 {
-    return Utils::anyOf(exprList, [&filePath](QRegExp reg) {
-        return (reg.exactMatch(filePath)
-                || reg.exactMatch(Utils::FileName::fromString(filePath).fileName()));
+    return Utils::anyOf(exprList, [&filePath](const QRegularExpression &reg) {
+        return (reg.match(filePath).hasMatch()
+                || reg.match(FilePath::fromString(filePath).fileName()).hasMatch());
     });
 }
 
-static bool isFileIncluded(const QList<QRegExp> &filterRegs, const QList<QRegExp> &exclusionRegs,
+static bool isFileIncluded(const QList<QRegularExpression> &filterRegs,
+                           const QList<QRegularExpression> &exclusionRegs,
                            const QString &filePath)
 {
     const bool isIncluded = filterRegs.isEmpty() || matches(filterRegs, filePath);
@@ -485,8 +508,8 @@ static bool isFileIncluded(const QList<QRegExp> &filterRegs, const QList<QRegExp
 std::function<bool(const QString &)>
 filterFileFunction(const QStringList &filters, const QStringList &exclusionFilters)
 {
-    const QList<QRegExp> filterRegs = filtersToRegExps(filters);
-    const QList<QRegExp> exclusionRegs = filtersToRegExps(exclusionFilters);
+    const QList<QRegularExpression> filterRegs = filtersToRegExps(filters);
+    const QList<QRegularExpression> exclusionRegs = filtersToRegExps(exclusionFilters);
     return [filterRegs, exclusionRegs](const QString &filePath) {
         return isFileIncluded(filterRegs, exclusionRegs, filePath);
     };
@@ -495,8 +518,8 @@ filterFileFunction(const QStringList &filters, const QStringList &exclusionFilte
 std::function<QStringList(const QStringList &)>
 filterFilesFunction(const QStringList &filters, const QStringList &exclusionFilters)
 {
-    const QList<QRegExp> filterRegs = filtersToRegExps(filters);
-    const QList<QRegExp> exclusionRegs = filtersToRegExps(exclusionFilters);
+    const QList<QRegularExpression> filterRegs = filtersToRegExps(filters);
+    const QList<QRegularExpression> exclusionRegs = filtersToRegExps(exclusionFilters);
     return [filterRegs, exclusionRegs](const QStringList &filePaths) {
         return Utils::filtered(filePaths, [&filterRegs, &exclusionRegs](const QString &filePath) {
             return isFileIncluded(filterRegs, exclusionRegs, filePath);
@@ -638,13 +661,18 @@ SubDirFileIterator::SubDirFileIterator(const QStringList &directories, const QSt
     : m_filterFiles(filterFilesFunction(filters, exclusionFilters)),
       m_progress(0)
 {
-    m_encoding = (encoding == 0 ? QTextCodec::codecForLocale() : encoding);
+    m_encoding = (encoding == nullptr ? QTextCodec::codecForLocale() : encoding);
     qreal maxPer = qreal(MAX_PROGRESS) / directories.count();
-    foreach (const QString &directoryEntry, directories) {
+    for (const QString &directoryEntry : directories) {
         if (!directoryEntry.isEmpty()) {
-            m_dirs.push(QDir(directoryEntry));
-            m_progressValues.push(maxPer);
-            m_processedValues.push(false);
+            const QDir dir(directoryEntry);
+            const QString canonicalPath = dir.canonicalPath();
+            if (!canonicalPath.isEmpty() && dir.exists()) {
+                m_dirs.push(dir);
+                m_knownDirs.insert(canonicalPath);
+                m_progressValues.push(maxPer);
+                m_processedValues.push(false);
+            }
         }
     }
 }
@@ -665,10 +693,18 @@ void SubDirFileIterator::update(int index)
         const bool processed = m_processedValues.pop();
         if (dir.exists()) {
             const QString dirPath = dir.path();
-            QStringList subDirs;
-            if (!processed)
-                subDirs = dir.entryList(QDir::Dirs|QDir::Hidden|QDir::NoDotAndDotDot);
-            if (subDirs.isEmpty()) {
+            using Dir = QString;
+            using CanonicalDir = QString;
+            std::vector<std::pair<Dir, CanonicalDir>> subDirs;
+            if (!processed) {
+                for (const QFileInfo &info :
+                     dir.entryInfoList(QDir::Dirs | QDir::Hidden | QDir::NoDotAndDotDot)) {
+                    const QString canonicalDir = info.canonicalFilePath();
+                    if (!m_knownDirs.contains(canonicalDir))
+                        subDirs.emplace_back(info.filePath(), canonicalDir);
+                }
+            }
+            if (subDirs.empty()) {
                 const QStringList allFileEntries = dir.entryList(QDir::Files|QDir::Hidden);
                 const QStringList allFilePaths = Utils::transform(allFileEntries,
                                                                   [&dirPath](const QString &entry) {
@@ -685,14 +721,13 @@ void SubDirFileIterator::update(int index)
                 m_dirs.push(dir);
                 m_progressValues.push(subProgress);
                 m_processedValues.push(true);
-                QStringListIterator it(subDirs);
-                it.toBack();
-                while (it.hasPrevious()) {
-                    const QString &directory = it.previous();
-                    m_dirs.push(QDir(dirPath + QLatin1Char('/') + directory));
-                    m_progressValues.push(subProgress);
-                    m_processedValues.push(false);
-                }
+                Utils::reverseForeach(subDirs,
+                                      [this, subProgress](const std::pair<Dir, CanonicalDir> &dir) {
+                                          m_dirs.push(QDir(dir.first));
+                                          m_knownDirs.insert(dir.second);
+                                          m_progressValues.push(subProgress);
+                                          m_processedValues.push(false);
+                                      });
             }
         } else {
             m_progress += dirProgressMax;
