@@ -35,11 +35,14 @@
 #include <qtsupport/qtsupportconstants.h>
 #include <qtsupport/qtversionmanager.h>
 
+#include <projectexplorer/buildconfiguration.h>
 #include <projectexplorer/projectexplorerconstants.h>
+#include <projectexplorer/projectnodes.h>
 #include <projectexplorer/target.h>
 #include <projectexplorer/kit.h>
 #include <projectexplorer/project.h>
 #include <projectexplorer/projecttree.h>
+#include <projectexplorer/buildsystem.h>
 
 #include <proparser/profileevaluator.h>
 
@@ -51,7 +54,7 @@ namespace Android {
 namespace Internal {
 
 AndroidQtVersion::AndroidQtVersion()
-    : QtSupport::BaseQtVersion()
+    : QtSupport::QtVersion()
     , m_guard(std::make_unique<QObject>())
 {
     QObject::connect(AndroidConfigurations::instance(),
@@ -62,7 +65,7 @@ AndroidQtVersion::AndroidQtVersion()
 
 bool AndroidQtVersion::isValid() const
 {
-    if (!BaseQtVersion::isValid())
+    if (!QtVersion::isValid())
         return false;
     if (qtAbis().isEmpty())
         return false;
@@ -71,7 +74,7 @@ bool AndroidQtVersion::isValid() const
 
 QString AndroidQtVersion::invalidReason() const
 {
-    QString tmp = BaseQtVersion::invalidReason();
+    QString tmp = QtVersion::invalidReason();
     if (tmp.isEmpty()) {
         if (AndroidConfigurations::currentConfig().ndkLocation(this).isEmpty())
             return tr("NDK is not configured in Devices > Android.");
@@ -84,6 +87,12 @@ QString AndroidQtVersion::invalidReason() const
     return tmp;
 }
 
+bool AndroidQtVersion::supportsMultipleQtAbis() const
+{
+    return qtVersion() >= QtSupport::QtVersionNumber{5, 14}
+           && qtVersion() < QtSupport::QtVersionNumber{6, 0};
+}
+
 Abis AndroidQtVersion::detectQtAbis() const
 {
     auto androidAbi2Abi = [](const QString &androidAbi) {
@@ -127,7 +136,7 @@ Abis AndroidQtVersion::detectQtAbis() const
 
 void AndroidQtVersion::addToEnvironment(const Kit *k, Utils::Environment &env) const
 {
-    BaseQtVersion::addToEnvironment(k, env);
+    QtVersion::addToEnvironment(k, env);
 
     const AndroidConfig &config = AndroidConfigurations::currentConfig();
     // this env vars are used by qmake mkspecs to generate makefiles (check QTDIR/mkspecs/android-g++/qmake.conf for more info)
@@ -137,12 +146,10 @@ void AndroidQtVersion::addToEnvironment(const Kit *k, Utils::Environment &env) c
             config.bestNdkPlatformMatch(qMax(minimumNDK(), AndroidManager::minimumSDK(k)), this));
 }
 
-Utils::Environment AndroidQtVersion::qmakeRunEnvironment() const
+void AndroidQtVersion::setupQmakeRunEnvironment(Utils::Environment &env) const
 {
-    Utils::Environment env = Utils::Environment::systemEnvironment();
     env.set(QLatin1String("ANDROID_NDK_ROOT"),
             AndroidConfigurations::currentConfig().ndkLocation(this).toUserOutput());
-    return env;
 }
 
 QString AndroidQtVersion::description() const
@@ -163,12 +170,34 @@ int AndroidQtVersion::minimumNDK() const
     return m_minNdk;
 }
 
+Utils::FilePath AndroidQtVersion::androidDeploymentSettings(const Target *target)
+{
+    // Try to fetch the file name from node data as provided by qmake and Qbs
+    QString buildKey = target->activeBuildKey();
+    const ProjectNode *node = target->project()->findNodeForBuildKey(buildKey);
+    if (node) {
+        const QString nameFromData = node->data(Constants::AndroidDeploySettingsFile).toString();
+        if (!nameFromData.isEmpty())
+            return Utils::FilePath::fromUserInput(nameFromData);
+    }
+
+    // If unavailable, construct the name by ourselves (CMake)
+    const BuildSystem *bs = target->buildSystem();
+    if (!bs)
+        return {};
+    const QString displayName = bs->buildTarget(buildKey).displayName;
+    return AndroidManager::buildDirectory(target).pathAppended(
+                AndroidManager::isQt5CmakeProject(target)
+                ? QLatin1String("android_deployment_settings.json")
+                : QString::fromLatin1("android-%1-deployment-settings.json")
+                  .arg(displayName));
+}
+
 void AndroidQtVersion::parseMkSpec(ProFileEvaluator *evaluator) const
 {
-    if (qtVersion() >= QtSupport::QtVersionNumber{5, 14})
-        m_androidAbis = evaluator->values("ALL_ANDROID_ABIS");
-    else
-        m_androidAbis = QStringList{evaluator->value("ANDROID_TARGET_ARCH")};
+    m_androidAbis = evaluator->values("ALL_ANDROID_ABIS");
+    if (m_androidAbis.isEmpty())
+        m_androidAbis = QStringList{evaluator->value(Constants::ANDROID_TARGET_ARCH)};
     const QString androidPlatform = evaluator->value("ANDROID_PLATFORM");
     if (!androidPlatform.isEmpty()) {
         const QRegularExpression regex("android-(\\d+)");
@@ -180,12 +209,12 @@ void AndroidQtVersion::parseMkSpec(ProFileEvaluator *evaluator) const
                 m_minNdk = tmp;
         }
     }
-    BaseQtVersion::parseMkSpec(evaluator);
+    QtVersion::parseMkSpec(evaluator);
 }
 
 QSet<Utils::Id> AndroidQtVersion::availableFeatures() const
 {
-    QSet<Utils::Id> features = QtSupport::BaseQtVersion::availableFeatures();
+    QSet<Utils::Id> features = QtSupport::QtVersion::availableFeatures();
     features.insert(QtSupport::Constants::FEATURE_MOBILE);
     features.remove(QtSupport::Constants::FEATURE_QT_CONSOLE);
     features.remove(QtSupport::Constants::FEATURE_QT_WEBKIT);
@@ -203,7 +232,7 @@ QSet<Utils::Id> AndroidQtVersion::targetDeviceTypes() const
 AndroidQtVersionFactory::AndroidQtVersionFactory()
 {
     setQtVersionCreator([] { return new AndroidQtVersion; });
-    setSupportedType(Constants::ANDROIDQT);
+    setSupportedType(Constants::ANDROID_QT_TYPE);
     setPriority(90);
 
     setRestrictionChecker([](const SetupData &setup) {
