@@ -30,11 +30,12 @@
 #include <widgethost.h>
 #include <designer/cpp/formclasswizardpage.h>
 
-#include <cpptools/cppmodelmanager.h>
-#include <cpptools/cpptoolsconstants.h>
-#include <cpptools/cppworkingcopy.h>
-#include <cpptools/insertionpointlocator.h>
-#include <cpptools/symbolfinder.h>
+#include <cppeditor/cppeditorconstants.h>
+#include <cppeditor/cppmodelmanager.h>
+#include <cppeditor/cpptoolsreuse.h>
+#include <cppeditor/cppworkingcopy.h>
+#include <cppeditor/insertionpointlocator.h>
+#include <cppeditor/symbolfinder.h>
 #include <cplusplus/LookupContext.h>
 #include <cplusplus/Overview.h>
 #include <coreplugin/icore.h>
@@ -42,9 +43,12 @@
 #include <texteditor/texteditor.h>
 #include <texteditor/textdocument.h>
 #include <projectexplorer/projectexplorer.h>
+#include <projectexplorer/projecttree.h>
 #include <projectexplorer/session.h>
+
 #include <utils/mimetypes/mimedatabase.h>
 #include <utils/qtcassert.h>
+#include <utils/stringutils.h>
 
 #include <QDesignerFormWindowInterface>
 #include <QDesignerFormEditorInterface>
@@ -62,12 +66,13 @@ using namespace Designer::Internal;
 using namespace CPlusPlus;
 using namespace TextEditor;
 using namespace ProjectExplorer;
+using namespace Utils;
 
 static QString msgClassNotFound(const QString &uiClassName, const QList<Document::Ptr> &docList)
 {
     QString files;
-    foreach (const Document::Ptr &doc, docList) {
-        files += QLatin1Char('\n');
+    for (const Document::Ptr &doc : docList) {
+        files += '\n';
         files += QDir::toNativeSeparators(doc->fileName());
     }
     return QtCreatorIntegration::tr(
@@ -85,8 +90,7 @@ QtCreatorIntegration::QtCreatorIntegration(QDesignerFormEditorInterface *core, Q
     f &= ~ResourceEditorFeature;
     setFeatures(f);
 
-    connect(this, static_cast<void (QDesignerIntegrationInterface::*)
-                    (const QString&, const QString&, const QStringList&)>
+    connect(this, QOverload<const QString &, const QString &, const QStringList &>::of
                        (&QDesignerIntegrationInterface::navigateToSlot),
             this, &QtCreatorIntegration::slotNavigateToSlot);
     connect(this, &QtCreatorIntegration::helpRequested,
@@ -114,17 +118,17 @@ QWidget *QtCreatorIntegration::containerWindow(QWidget * /*widget*/) const
 {
     if (SharedTools::WidgetHost *host = FormEditorW::activeWidgetHost())
         return host->integrationContainer();
-    return 0;
+    return nullptr;
 }
 
 static QList<Document::Ptr> findDocumentsIncluding(const Snapshot &docTable,
                                                    const QString &fileName, bool checkFileNameOnly)
 {
     QList<Document::Ptr> docList;
-    foreach (const Document::Ptr &doc, docTable) { // we go through all documents
+    for (const Document::Ptr &doc : docTable) { // we go through all documents
         const QList<Document::Include> includes = doc->resolvedIncludes()
             + doc->unresolvedIncludes();
-        foreach (const Document::Include &include, includes) {
+        for (const Document::Include &include : includes) {
             if (checkFileNameOnly) {
                 const QFileInfo fi(include.unresolvedFileName());
                 if (fi.fileName() == fileName) { // we are only interested in docs which includes fileName only
@@ -143,7 +147,7 @@ static QList<Document::Ptr> findDocumentsIncluding(const Snapshot &docTable,
 static bool inherits(const Overview &o, const Class *klass, const QString &baseClass)
 {
     const int baseClassCount = klass->baseClassCount();
-    for (int b = 0; b < baseClassCount; b++)
+    for (int b = 0; b < baseClassCount; ++b)
         if (o.prettyName(klass->baseClassAt(b)->name()) == baseClass)
             return true;
     return false;
@@ -155,33 +159,30 @@ QString fullyQualifiedName(const LookupContext &context, const Name *name, Scope
         return QString();
 
     const QList<LookupItem> items = context.lookup(name, scope);
-    if (items.isEmpty()) { // "ui_xxx.h" might not be generated and nothing is forward declared.
+    if (items.isEmpty()) // "ui_xxx.h" might not be generated and nothing is forward declared.
         return Overview().prettyName(name);
-    } else {
-        Symbol *symbol = items.first().declaration();
-        return Overview().prettyName(LookupContext::fullyQualifiedName(symbol));
-    }
-    return QString();
+    Symbol *symbol = items.first().declaration();
+    return Overview().prettyName(LookupContext::fullyQualifiedName(symbol));
 }
 
 // Find class definition in namespace (that is, the outer class
 // containing a member of the desired class type) or inheriting the desired class
 // in case of forms using the Multiple Inheritance approach
 static const Class *findClass(const Namespace *parentNameSpace, const LookupContext &context,
-                              const QString &className, QString *namespaceName)
+                              const QString &className)
 {
     if (Designer::Constants::Internal::debug)
         qDebug() << Q_FUNC_INFO << className;
 
     const Overview o;
-    const unsigned namespaceMemberCount = parentNameSpace->memberCount();
-    for (unsigned i = 0; i < namespaceMemberCount; ++i) { // we go through all namespace members
+    const int namespaceMemberCount = parentNameSpace->memberCount();
+    for (int i = 0; i < namespaceMemberCount; ++i) { // we go through all namespace members
         const Symbol *sym = parentNameSpace->memberAt(i);
         // we have found a class - we are interested in classes only
         if (const Class *cl = sym->asClass()) {
             // 1) we go through class members
-            const unsigned classMemberCount = cl->memberCount();
-            for (unsigned j = 0; j < classMemberCount; ++j)
+            const int classMemberCount = cl->memberCount();
+            for (int j = 0; j < classMemberCount; ++j)
                 if (Declaration *decl = cl->memberAt(j)->asDeclaration()) {
                 // we want to know if the class contains a member (so we look into
                 // a declaration) of uiClassName type
@@ -205,70 +206,50 @@ static const Class *findClass(const Namespace *parentNameSpace, const LookupCont
         } else {
             // Check namespaces
             if (const Namespace *ns = sym->asNamespace()) {
-                QString tempNS = *namespaceName;
-                tempNS += o.prettyName(ns->name());
-                tempNS += QLatin1String("::");
-                if (const Class *cl = findClass(ns, context, className, &tempNS)) {
-                    *namespaceName = tempNS;
+                if (const Class *cl = findClass(ns, context, className))
                     return cl;
-                }
-            } // member is namespave
+            } // member is namespace
         } // member is no class
     } // for members
-    return 0;
+    return nullptr;
 }
 
 static Function *findDeclaration(const Class *cl, const QString &functionName)
 {
     const QString funName = QString::fromUtf8(QMetaObject::normalizedSignature(functionName.toUtf8()));
-    const unsigned mCount = cl->memberCount();
+    const int mCount = cl->memberCount();
     // we are interested only in declarations (can be decl of function or of a field)
     // we are only interested in declarations of functions
     const Overview overview;
-    for (unsigned j = 0; j < mCount; ++j) { // go through all members
+    for (int j = 0; j < mCount; ++j) { // go through all members
         if (Declaration *decl = cl->memberAt(j)->asDeclaration())
             if (Function *fun = decl->type()->asFunctionType()) {
                 // Format signature
                 QString memberFunction = overview.prettyName(fun->name());
-                memberFunction += QLatin1Char('(');
-                const uint aCount = fun->argumentCount();
-                for (uint i = 0; i < aCount; i++) { // we build argument types string
+                memberFunction += '(';
+                const int aCount = fun->argumentCount();
+                for (int i = 0; i < aCount; i++) { // we build argument types string
                     const Argument *arg = fun->argumentAt(i)->asArgument();
                     if (i > 0)
-                        memberFunction += QLatin1Char(',');
+                        memberFunction += ',';
                     memberFunction += overview.prettyType(arg->type());
                 }
-                memberFunction += QLatin1Char(')');
+                memberFunction += ')';
                 // we compare normalized signatures
                 memberFunction = QString::fromUtf8(QMetaObject::normalizedSignature(memberFunction.toUtf8()));
                 if (memberFunction == funName) // we match function names and argument lists
                     return fun;
             }
     }
-    return 0;
-}
-
-// TODO: remove me, this is taken from cppeditor.cpp. Find some common place for this function
-static Document::Ptr findDefinition(Function *functionDeclaration, int *line)
-{
-    CppTools::CppModelManager *cppModelManager = CppTools::CppModelManager::instance();
-    const Snapshot snapshot = cppModelManager->snapshot();
-    CppTools::SymbolFinder symbolFinder;
-    if (Function *fun = symbolFinder.findMatchingDefinition(functionDeclaration, snapshot)) {
-        if (line)
-            *line = fun->line();
-
-        return snapshot.document(QString::fromUtf8(fun->fileName(), fun->fileNameLength()));
-    }
-
-    return Document::Ptr();
+    return nullptr;
 }
 
 static inline BaseTextEditor *editorAt(const QString &fileName, int line, int column)
 {
-    return qobject_cast<BaseTextEditor *>(Core::EditorManager::openEditorAt(fileName, line, column,
-                                                                         Core::Id(),
-                                                                         Core::EditorManager::DoNotMakeVisible));
+    return qobject_cast<BaseTextEditor *>(
+        Core::EditorManager::openEditorAt({FilePath::fromString(fileName), line, column},
+                                          Utils::Id(),
+                                          Core::EditorManager::DoNotMakeVisible));
 }
 
 static void addDeclaration(const Snapshot &snapshot,
@@ -276,14 +257,12 @@ static void addDeclaration(const Snapshot &snapshot,
                            const Class *cl,
                            const QString &functionName)
 {
-    QString declaration = QLatin1String("void ");
-    declaration += functionName;
-    declaration += QLatin1String(";\n");
+    const QString declaration = "void " + functionName + ";\n";
 
-    CppTools::CppRefactoringChanges refactoring(snapshot);
-    CppTools::InsertionPointLocator find(refactoring);
-    const CppTools::InsertionLocation loc = find.methodDeclarationInClass(
-                fileName, cl, CppTools::InsertionPointLocator::PrivateSlot);
+    CppEditor::CppRefactoringChanges refactoring(snapshot);
+    CppEditor::InsertionPointLocator find(refactoring);
+    const CppEditor::InsertionLocation loc = find.methodDeclarationInClass(
+                fileName, cl, CppEditor::InsertionPointLocator::PrivateSlot);
 
     //
     //! \todo change this to use the Refactoring changes.
@@ -300,82 +279,22 @@ static void addDeclaration(const Snapshot &snapshot,
     }
 }
 
-static Document::Ptr addDefinition(const Snapshot &docTable,
-                                   const QString &headerFileName,
-                                   const QString &className,
-                                   const QString &functionName,
-                                   int *line)
-{
-    QString definition = QLatin1String("\nvoid ");
-    definition += className;
-    definition += QLatin1String("::");
-    definition += functionName;
-    definition += QLatin1String("\n{\n");
-    definition += QString(indentation, QLatin1Char(' '));
-    definition += QLatin1String("\n}\n");
-
-    // we find all documents which include headerFileName
-    const QList<Document::Ptr> docList = findDocumentsIncluding(docTable, headerFileName, false);
-    if (docList.isEmpty())
-        return Document::Ptr();
-
-    QFileInfo headerFI(headerFileName);
-    const QString headerBaseName = headerFI.completeBaseName();
-    foreach (const Document::Ptr &doc, docList) {
-        const QFileInfo sourceFI(doc->fileName());
-        // we take only those documents which have the same filename
-        if (headerBaseName == sourceFI.baseName()) {
-            //
-            //! \todo change this to use the Refactoring changes.
-            //
-
-            if (BaseTextEditor *editor = editorAt(doc->fileName(), 0, 0)) {
-
-                //
-                //! \todo use the InsertionPointLocator to insert at the correct place.
-                // (we'll have to extend that class first to do definition insertions)
-
-                const QString contents = editor->textDocument()->plainText();
-                int column;
-                editor->convertPosition(contents.length(), line, &column);
-                editor->gotoLine(*line, column);
-                editor->insert(definition);
-                *line += 1;
-            }
-            return doc;
-        }
-    }
-    return Document::Ptr();
-}
-
 static QString addConstRefIfNeeded(const QString &argument)
 {
-    if (argument.startsWith(QLatin1String("const "))
-            || argument.endsWith(QLatin1Char('&'))
-            || argument.endsWith(QLatin1Char('*')))
+    if (argument.startsWith("const ") || argument.endsWith('&') || argument.endsWith('*'))
         return argument;
 
     // for those types we don't want to add "const &"
-    static const QStringList nonConstRefs = QStringList()
-            << QLatin1String("bool")
-            << QLatin1String("int")
-            << QLatin1String("uint")
-            << QLatin1String("float")
-            << QLatin1String("double")
-            << QLatin1String("long")
-            << QLatin1String("short")
-            << QLatin1String("char")
-            << QLatin1String("signed")
-            << QLatin1String("unsigned")
-            << QLatin1String("qint64")
-            << QLatin1String("quint64");
+    static const QStringList nonConstRefs = QStringList({"bool", "int", "uint", "float", "double",
+                                                         "long", "short", "char", "signed",
+                                                         "unsigned", "qint64", "quint64"});
 
     for (int i = 0; i < nonConstRefs.count(); i++) {
-        const QString nonConstRef = nonConstRefs.at(i);
-        if (argument == nonConstRef || argument.startsWith(nonConstRef + QLatin1Char(' ')))
+        const QString &nonConstRef = nonConstRefs.at(i);
+        if (argument == nonConstRef || argument.startsWith(nonConstRef + ' '))
             return argument;
     }
-    return QLatin1String("const ") + argument + QLatin1Char('&');
+    return "const " + argument + '&';
 }
 
 static QString formatArgument(const QString &argument)
@@ -385,8 +304,8 @@ static QString formatArgument(const QString &argument)
     while (i > 0) { // from the end of the "argument" string
         i--;
         const QChar c = argument.at(i); // take the char
-        if (c != QLatin1Char('*') && c != QLatin1Char('&')) { // if it's not the * or &
-            formattedArgument.insert(i + 1, QLatin1Char(' ')); // insert space after that char or just append space (to separate it from the parameter name)
+        if (c != '*' && c != '&') { // if it's not the * or &
+            formattedArgument.insert(i + 1, ' '); // insert space after that char or just append space (to separate it from the parameter name)
             break;
         }
     }
@@ -397,25 +316,25 @@ static QString formatArgument(const QString &argument)
 // "void foo(bool checked)"
 static QString addParameterNames(const QString &functionSignature, const QStringList &parameterNames)
 {
-    const int firstParen = functionSignature.indexOf(QLatin1Char('('));
+    const int firstParen = functionSignature.indexOf('(');
     QString functionName = functionSignature.left(firstParen + 1);
     QString argumentsString = functionSignature.mid(firstParen + 1);
-    const int lastParen = argumentsString.lastIndexOf(QLatin1Char(')'));
+    const int lastParen = argumentsString.lastIndexOf(')');
     if (lastParen != -1)
         argumentsString.truncate(lastParen);
-    const QStringList arguments = argumentsString.split(QLatin1Char(','), QString::SkipEmptyParts);
+    const QStringList arguments = argumentsString.split(',', Qt::SkipEmptyParts);
     const int pCount = parameterNames.count();
     const int aCount = arguments.count();
     for (int i = 0; i < aCount; ++i) {
         if (i > 0)
-            functionName += QLatin1String(", ");
+            functionName += ", ";
         const QString argument = addConstRefIfNeeded(arguments.at(i));
         functionName += formatArgument(argument);
         if (i < pCount) {
             // prepare parameterName
             QString parameterName = parameterNames.at(i);
             if (parameterName.isEmpty()) {
-                const QString generatedName = QLatin1String("arg") + QString::number(i + 1);
+                const QString generatedName = "arg" + QString::number(i + 1);
                 if (!parameterNames.contains(generatedName))
                     parameterName = generatedName;
             }
@@ -424,7 +343,7 @@ static QString addParameterNames(const QString &functionSignature, const QString
                 functionName += parameterName;
         }
     }
-    functionName += QLatin1Char(')');
+    functionName += ')';
     return functionName;
 }
 
@@ -432,29 +351,30 @@ static QString addParameterNames(const QString &functionSignature, const QString
 // included files (going down [maxIncludeDepth] includes) and return a pair
 // of <Class*, Document>.
 
-typedef QPair<const Class *, Document::Ptr> ClassDocumentPtrPair;
+using ClassDocumentPtrPair = QPair<const Class *, Document::Ptr>;
 
 static ClassDocumentPtrPair
         findClassRecursively(const LookupContext &context, const QString &className,
-                             unsigned maxIncludeDepth, QString *namespaceName)
+                             unsigned maxIncludeDepth)
 {
     const Document::Ptr doc = context.thisDocument();
     const Snapshot docTable = context.snapshot();
     if (Designer::Constants::Internal::debug)
         qDebug() << Q_FUNC_INFO << doc->fileName() << className << maxIncludeDepth;
     // Check document
-    if (const Class *cl = findClass(doc->globalNamespace(), context, className, namespaceName))
+    if (const Class *cl = findClass(doc->globalNamespace(), context, className))
         return ClassDocumentPtrPair(cl, doc);
     if (maxIncludeDepth) {
         // Check the includes
         const unsigned recursionMaxIncludeDepth = maxIncludeDepth - 1u;
-        foreach (const QString &include, doc->includedFiles()) {
+        const auto includedFiles = doc->includedFiles();
+        for (const QString &include : includedFiles) {
             const Snapshot::const_iterator it = docTable.find(include);
             if (it != docTable.end()) {
-                const Document::Ptr includeDoc = it.value();
+                const Document::Ptr &includeDoc = it.value();
                 LookupContext context(includeDoc, docTable);
                 const ClassDocumentPtrPair irc = findClassRecursively(context, className,
-                    recursionMaxIncludeDepth, namespaceName);
+                    recursionMaxIncludeDepth);
                 if (irc.first)
                     return irc;
             }
@@ -474,16 +394,18 @@ void QtCreatorIntegration::slotNavigateToSlot(const QString &objectName, const Q
 // Build name of the class as generated by uic, insert Ui namespace
 // "foo::bar::form" -> "foo::bar::Ui::form"
 
-static inline QString uiClassName(QString formObjectName)
+static inline const QStringList uiClassNames(QString formObjectName)
 {
-    const int indexOfScope = formObjectName.lastIndexOf(QLatin1String("::"));
+    const int indexOfScope = formObjectName.lastIndexOf("::");
     const int uiNameSpaceInsertionPos = indexOfScope >= 0 ? indexOfScope + 2 : 0;
-    formObjectName.insert(uiNameSpaceInsertionPos, QLatin1String("Ui::"));
-    return formObjectName;
+    QString alt = formObjectName;
+    formObjectName.insert(uiNameSpaceInsertionPos, "Ui::");
+    alt.insert(uiNameSpaceInsertionPos, "Ui_");
+    return {formObjectName, alt};
 }
 
 static Document::Ptr getParsedDocument(const QString &fileName,
-                                       CppTools::WorkingCopy &workingCopy,
+                                       CppEditor::WorkingCopy &workingCopy,
                                        Snapshot &snapshot)
 {
     QByteArray src;
@@ -491,11 +413,11 @@ static Document::Ptr getParsedDocument(const QString &fileName,
         src = workingCopy.source(fileName);
     } else {
         Utils::FileReader reader;
-        if (reader.fetch(fileName)) // ### FIXME error reporting
+        if (reader.fetch(Utils::FilePath::fromString(fileName))) // ### FIXME error reporting
             src = QString::fromLocal8Bit(reader.data()).toUtf8();
     }
 
-    Document::Ptr doc = snapshot.preprocessedDocument(src, fileName);
+    Document::Ptr doc = snapshot.preprocessedDocument(src, FilePath::fromString(fileName));
     doc->check();
     snapshot.insert(doc);
     return doc;
@@ -509,9 +431,9 @@ bool QtCreatorIntegration::navigateToSlot(const QString &objectName,
                                           const QStringList &parameterNames,
                                           QString *errorMessage)
 {
-    typedef QMap<int, Document::Ptr> DocumentMap;
+    using DocumentMap = QMap<int, Document::Ptr>;
 
-    const Utils::FileName currentUiFile = FormEditorW::activeEditor()->document()->filePath();
+    const Utils::FilePath currentUiFile = FormEditorW::activeEditor()->document()->filePath();
 #if 0
     return Designer::Internal::navigateToSlot(currentUiFile.toString(), objectName,
                                               signalSignature, parameterNames, errorMessage);
@@ -523,10 +445,10 @@ bool QtCreatorIntegration::navigateToSlot(const QString &objectName,
     // So we should somehow get that info from project manager (?)
     const QFileInfo fi = currentUiFile.toFileInfo();
     const QString uiFolder = fi.absolutePath();
-    const QString uicedName = QLatin1String("ui_") + fi.completeBaseName() + QLatin1String(".h");
+    const QString uicedName = "ui_" + fi.completeBaseName() + ".h";
 
     // Retrieve code model snapshot restricted to project of ui file or the working copy.
-    Snapshot docTable = CppTools::CppModelManager::instance()->snapshot();
+    Snapshot docTable = CppEditor::CppModelManager::instance()->snapshot();
     Snapshot newDocTable;
     const Project *uiProject = SessionManager::projectForFile(currentUiFile);
     if (uiProject) {
@@ -536,14 +458,12 @@ bool QtCreatorIntegration::navigateToSlot(const QString &objectName,
                 newDocTable.insert(i.value());
         }
     } else {
-        const CppTools::WorkingCopy workingCopy =
-                CppTools::CppModelManager::instance()->workingCopy();
-        const Utils::FileName configFileName =
-                Utils::FileName::fromString(CppTools::CppModelManager::configurationFileName());
-        QHashIterator<Utils::FileName, QPair<QByteArray, unsigned> > it = workingCopy.iterator();
-        while (it.hasNext()) {
-            it.next();
-            const Utils::FileName &fileName = it.key();
+        const Utils::FilePath configFileName =
+                Utils::FilePath::fromString(CppEditor::CppModelManager::configurationFileName());
+        const CppEditor::WorkingCopy::Table elements =
+                CppEditor::CppModelManager::instance()->workingCopy().elements();
+        for (auto it = elements.cbegin(), end = elements.cend(); it != end; ++it) {
+            const Utils::FilePath &fileName = it.key();
             if (fileName != configFileName)
                 newDocTable.insert(docTable.document(fileName));
         }
@@ -555,7 +475,7 @@ bool QtCreatorIntegration::navigateToSlot(const QString &objectName,
     // first in case there are project subdirectories that contain identical file names.
     const QList<Document::Ptr> docList = findDocumentsIncluding(docTable, uicedName, true); // change to false when we know the absolute path to generated ui_<>.h file
     DocumentMap docMap;
-    foreach (const Document::Ptr &d, docList) {
+    for (const Document::Ptr &d : docList) {
         const QFileInfo docFi(d->fileName());
         docMap.insert(qAbs(docFi.absolutePath().compare(uiFolder, Qt::CaseInsensitive)), d);
     }
@@ -569,78 +489,106 @@ bool QtCreatorIntegration::navigateToSlot(const QString &objectName,
 
     QDesignerFormWindowInterface *fwi = FormEditorW::activeWidgetHost()->formWindow();
 
-    const QString uiClass = uiClassName(fwi->mainContainer()->objectName());
+    QString uiClass;
+    const Class *cl = nullptr;
+    Document::Ptr declDoc;
+    for (const QString &candidate : uiClassNames(fwi->mainContainer()->objectName())) {
+        if (Designer::Constants::Internal::debug)
+            qDebug() << "Checking docs for " << candidate;
 
-    if (Designer::Constants::Internal::debug)
-        qDebug() << "Checking docs for " << uiClass;
-
-    // Find the class definition (ui class defined as member or base class)
-    // in the file itself or in the directly included files (order 1).
-    QString namespaceName;
-    const Class *cl = 0;
-    Document::Ptr doc;
-
-    foreach (const Document::Ptr &d, docMap) {
-        LookupContext context(d, docTable);
-        const ClassDocumentPtrPair cd = findClassRecursively(context, uiClass, 1u , &namespaceName);
-        if (cd.first) {
-            cl = cd.first;
-            doc = cd.second;
+        // Find the class definition (ui class defined as member or base class)
+        // in the file itself or in the directly included files (order 1).
+        for (const Document::Ptr &d : qAsConst(docMap)) {
+            LookupContext context(d, docTable);
+            const ClassDocumentPtrPair cd = findClassRecursively(context, candidate, 1u);
+            if (cd.first) {
+                cl = cd.first;
+                declDoc = cd.second;
+                break;
+            }
+        }
+        if (cl) {
+            uiClass = candidate;
             break;
         }
+
+        if (errorMessage->isEmpty())
+            *errorMessage = msgClassNotFound(candidate, docList);
     }
-    if (!cl) {
-        *errorMessage = msgClassNotFound(uiClass, docList);
+    if (!cl)
         return false;
-    }
 
-    Overview o;
-    const QString className = namespaceName + o.prettyName(cl->name());
-    if (Designer::Constants::Internal::debug)
-        qDebug() << "Found class  " << className << doc->fileName();
-
-    const QString functionName = QLatin1String("on_") + objectName + QLatin1Char('_') + signalSignature;
+    const QString functionName = "on_" + objectName + '_' + signalSignature;
     const QString functionNameWithParameterNames = addParameterNames(functionName, parameterNames);
 
     if (Designer::Constants::Internal::debug)
-        qDebug() << Q_FUNC_INFO << "Found " << uiClass << doc->fileName() << " checking " << functionName  << functionNameWithParameterNames;
+        qDebug() << Q_FUNC_INFO << "Found " << uiClass << declDoc->fileName() << " checking " << functionName  << functionNameWithParameterNames;
 
-    int line = 0;
-    Document::Ptr sourceDoc;
-
-    if (Function *fun = findDeclaration(cl, functionName)) {
-        sourceDoc = findDefinition(fun, &line);
-        if (!sourceDoc) {
-            // add function definition to cpp file
-            sourceDoc = addDefinition(docTable, doc->fileName(), className, functionNameWithParameterNames, &line);
-        }
-    } else {
+    Function *fun = findDeclaration(cl, functionName);
+    QString declFilePath;
+    if (!fun) {
         // add function declaration to cl
-        CppTools::WorkingCopy workingCopy =
-            CppTools::CppModelManager::instance()->workingCopy();
-        const QString fileName = doc->fileName();
-        getParsedDocument(fileName, workingCopy, docTable);
-        addDeclaration(docTable, fileName, cl, functionNameWithParameterNames);
+        CppEditor::WorkingCopy workingCopy = CppEditor::CppModelManager::instance()->workingCopy();
+        declFilePath = declDoc->fileName();
+        getParsedDocument(declFilePath, workingCopy, docTable);
+        addDeclaration(docTable, declFilePath, cl, functionNameWithParameterNames);
 
-        // add function definition to cpp file
-        sourceDoc = addDefinition(docTable, fileName, className, functionNameWithParameterNames, &line);
+        // Re-load C++ documents.
+        QList<Utils::FilePath> filePaths;
+        for (auto it = docTable.begin(); it != docTable.end(); ++it)
+            filePaths << it.key();
+        workingCopy = CppEditor::CppModelManager::instance()->workingCopy();
+        docTable = CppEditor::CppModelManager::instance()->snapshot();
+        newDocTable = {};
+        for (const auto &file : qAsConst(filePaths)) {
+            const Document::Ptr doc = docTable.document(file);
+            if (doc)
+                newDocTable.insert(doc);
+        }
+        docTable = newDocTable;
+        getParsedDocument(declFilePath, workingCopy, docTable);
+        const Document::Ptr headerDoc = docTable.document(declFilePath);
+        QTC_ASSERT(headerDoc, return false);
+        LookupContext context(headerDoc, docTable);
+        cl = findClass(headerDoc->globalNamespace(), context, uiClass);
+        QTC_ASSERT(cl, return false);
+        fun = findDeclaration(cl, functionName);
+    } else {
+        declFilePath = QLatin1String(fun->fileName());
+    }
+    QTC_ASSERT(fun, return false);
+
+    CppEditor::CppRefactoringChanges refactoring(docTable);
+    CppEditor::SymbolFinder symbolFinder;
+    if (const Function *funImpl = symbolFinder.findMatchingDefinition(fun, docTable, true)) {
+        Core::EditorManager::openEditorAt(
+            {FilePath::fromString(QString::fromUtf8(funImpl->fileName())), funImpl->line() + 2});
+        return true;
+    }
+    const QString implFilePath = CppEditor::correspondingHeaderOrSource(declFilePath);
+    const CppEditor::InsertionLocation location = CppEditor::insertLocationForMethodDefinition
+            (fun, false, CppEditor::NamespaceHandling::CreateMissing, refactoring, implFilePath);
+
+    if (BaseTextEditor *editor = editorAt(location.fileName(), location.line(), location.column())) {
+        Overview o;
+        const QString className = o.prettyName(cl->name());
+        const QString definition = location.prefix() + "void " + className + "::"
+            + functionNameWithParameterNames + "\n{\n" + QString(indentation, ' ') + "\n}\n"
+            + location.suffix();
+        editor->insert(definition);
+        Core::EditorManager::openEditorAt({FilePath::fromString(location.fileName()),
+                                           int(location.line() + location.prefix().count('\n') + 2),
+                                           indentation});
+        return true;
     }
 
-    if (!sourceDoc) {
-        *errorMessage = tr("Unable to add the method definition.");
-        return false;
-    }
-
-    // jump to function definition, position within code
-    Core::EditorManager::openEditorAt(sourceDoc->fileName(), line + 2, indentation);
-
-    return true;
+    *errorMessage = tr("Unable to add the method definition.");
+    return false;
 }
 
 void QtCreatorIntegration::slotSyncSettingsToDesigner()
 {
     // Set promotion-relevant parameters on integration.
-    Utils::MimeDatabase mdb;
-    setHeaderSuffix(mdb.mimeTypeForName(QLatin1String(CppTools::Constants::CPP_HEADER_MIMETYPE)).preferredSuffix());
+    setHeaderSuffix(Utils::mimeTypeForName(CppEditor::Constants::CPP_HEADER_MIMETYPE).preferredSuffix());
     setHeaderLowercase(FormClassWizardPage::lowercaseHeaderFiles());
 }
