@@ -32,6 +32,7 @@
 #include "target.h"
 
 #include <cmakeprojectmanager/cmakeprojectconstants.h>
+#include <docker/dockerconstants.h>
 #include <qbsprojectmanager/qbsprojectmanagerconstants.h>
 #include <qmakeprojectmanager/qmakeprojectmanagerconstants.h>
 
@@ -40,8 +41,6 @@
 #include <utils/qtcassert.h>
 #include <utils/stringutils.h>
 
-#include <QFileInfo>
-
 using namespace Utils;
 
 namespace ProjectExplorer {
@@ -54,17 +53,17 @@ class DesktopRunConfiguration : public RunConfiguration
 protected:
     enum Kind { Qmake, Qbs, CMake }; // FIXME: Remove
 
-    DesktopRunConfiguration(Target *target, Utils::Id id, Kind kind);
+    DesktopRunConfiguration(Target *target, Id id, Kind kind);
 
 private:
     void updateTargetInformation();
 
-    Utils::FilePath executableToRun(const BuildTargetInfo &targetInfo) const;
+    FilePath executableToRun(const BuildTargetInfo &targetInfo) const;
 
     const Kind m_kind;
 };
 
-DesktopRunConfiguration::DesktopRunConfiguration(Target *target, Utils::Id id, Kind kind)
+DesktopRunConfiguration::DesktopRunConfiguration(Target *target, Id id, Kind kind)
     : RunConfiguration(target, id), m_kind(kind)
 {
     auto envAspect = addAspect<LocalEnvironmentAspect>(target);
@@ -88,6 +87,9 @@ DesktopRunConfiguration::DesktopRunConfiguration(Target *target, Utils::Id id, K
         });
     }
 
+    if (HostOsInfo::isAnyUnixHost())
+        addAspect<RunAsRootAspect>();
+
     envAspect->addModifier([this, libAspect](Environment &env) {
         BuildTargetInfo bti = buildTargetInfo();
         if (bti.runEnvModifier)
@@ -116,9 +118,9 @@ void DesktopRunConfiguration::updateTargetInformation()
         if (profile.isEmpty())
             setDefaultDisplayName(tr("Qt Run Configuration"));
         else
-            setDefaultDisplayName(profile.toFileInfo().completeBaseName());
+            setDefaultDisplayName(profile.completeBaseName());
 
-        aspect<EnvironmentAspect>()->environmentChanged();
+        emit aspect<EnvironmentAspect>()->environmentChanged();
 
         auto wda = aspect<WorkingDirectoryAspect>();
         wda->setDefaultWorkingDirectory(bti.workingDirectory);
@@ -142,12 +144,12 @@ void DesktopRunConfiguration::updateTargetInformation()
 
         aspect<ExecutableAspect>()->setExecutable(bti.targetFilePath);
         aspect<WorkingDirectoryAspect>()->setDefaultWorkingDirectory(bti.workingDirectory);
-        aspect<LocalEnvironmentAspect>()->environmentChanged();
+        emit aspect<LocalEnvironmentAspect>()->environmentChanged();
 
     }
 }
 
-Utils::FilePath DesktopRunConfiguration::executableToRun(const BuildTargetInfo &targetInfo) const
+FilePath DesktopRunConfiguration::executableToRun(const BuildTargetInfo &targetInfo) const
 {
     const FilePath appInBuildDir = targetInfo.targetFilePath;
     const DeploymentData deploymentData = target()->deploymentData();
@@ -168,7 +170,7 @@ Utils::FilePath DesktopRunConfiguration::executableToRun(const BuildTargetInfo &
 class DesktopQmakeRunConfiguration final : public DesktopRunConfiguration
 {
 public:
-    DesktopQmakeRunConfiguration(Target *target, Utils::Id id)
+    DesktopQmakeRunConfiguration(Target *target, Id id)
         : DesktopRunConfiguration(target, id, Qmake)
     {}
 };
@@ -176,7 +178,7 @@ public:
 class QbsRunConfiguration final : public DesktopRunConfiguration
 {
 public:
-    QbsRunConfiguration(Target *target, Utils::Id id)
+    QbsRunConfiguration(Target *target, Id id)
         : DesktopRunConfiguration(target, id, Qbs)
     {}
 };
@@ -184,7 +186,7 @@ public:
 class CMakeRunConfiguration final : public DesktopRunConfiguration
 {
 public:
-    CMakeRunConfiguration(Target *target, Utils::Id id)
+    CMakeRunConfiguration(Target *target, Id id)
         : DesktopRunConfiguration(target, id, CMake)
     {}
 };
@@ -198,6 +200,7 @@ CMakeRunConfigurationFactory::CMakeRunConfigurationFactory()
     registerRunConfiguration<CMakeRunConfiguration>(CMAKE_RUNCONFIG_ID);
     addSupportedProjectType(CMakeProjectManager::Constants::CMAKE_PROJECT_ID);
     addSupportedTargetDeviceType(ProjectExplorer::Constants::DESKTOP_DEVICE_TYPE);
+    addSupportedTargetDeviceType(Docker::Constants::DOCKER_DEVICE_TYPE);
 }
 
 QbsRunConfigurationFactory::QbsRunConfigurationFactory()
@@ -205,6 +208,7 @@ QbsRunConfigurationFactory::QbsRunConfigurationFactory()
     registerRunConfiguration<QbsRunConfiguration>(QBS_RUNCONFIG_ID);
     addSupportedProjectType(QbsProjectManager::Constants::PROJECT_ID);
     addSupportedTargetDeviceType(ProjectExplorer::Constants::DESKTOP_DEVICE_TYPE);
+    addSupportedTargetDeviceType(Docker::Constants::DOCKER_DEVICE_TYPE);
 }
 
 DesktopQmakeRunConfigurationFactory::DesktopQmakeRunConfigurationFactory()
@@ -212,6 +216,7 @@ DesktopQmakeRunConfigurationFactory::DesktopQmakeRunConfigurationFactory()
     registerRunConfiguration<DesktopQmakeRunConfiguration>(QMAKE_RUNCONFIG_ID);
     addSupportedProjectType(QmakeProjectManager::Constants::QMAKEPROJECT_ID);
     addSupportedTargetDeviceType(ProjectExplorer::Constants::DESKTOP_DEVICE_TYPE);
+    addSupportedTargetDeviceType(Docker::Constants::DOCKER_DEVICE_TYPE);
 }
 
 } // namespace Internal
