@@ -42,23 +42,7 @@ using qbs::Profile;
 
 TestBlackboxQt::TestBlackboxQt() : TestBlackboxBase (SRCDIR "/testdata-qt", "blackbox-qt")
 {
-}
-
-void TestBlackboxQt::validateTestProfile()
-{
-    const SettingsPtr s = settings();
-    if (profileName() != "none" && !s->profiles().contains(profileName()))
-        QFAIL(QByteArray("The build profile '" + profileName().toLocal8Bit() +
-                         "' could not be found. Please set it up on your machine."));
-    const QStringList qmakeFilePaths = Profile(profileName(), s.get())
-            .value("moduleProviders.Qt.qmakeFilePaths").toStringList();
-    if (!qmakeFilePaths.empty())
-        return;
-    if (!findExecutable(QStringList{"qmake"}).isEmpty())
-        return;
-    QSKIP(QByteArray("The build profile '" + profileName().toLocal8Bit() +
-                     "' is not a valid Qt profile and Qt was not found "
-                     "in the global search paths."));
+    setNeedsQt();
 }
 
 void TestBlackboxQt::addQObjectMacroToGeneratedCppFile()
@@ -75,6 +59,9 @@ void TestBlackboxQt::addQObjectMacroToGeneratedCppFile()
 void TestBlackboxQt::autoQrc()
 {
     QDir::setCurrent(testDataDir + "/auto-qrc");
+    QCOMPARE(runQbs(QbsRunParameters("resolve")), 0);
+    if (m_qbsStdout.contains("targetPlatform differs from hostPlatform"))
+        QSKIP("Cannot run binaries in cross-compiled build");
     QCOMPARE(runQbs(QbsRunParameters("run", QStringList{"-p", "app"})), 0);
     QVERIFY2(m_qbsStdout.simplified().contains("resource data: resource1 resource2"),
              m_qbsStdout.constData());
@@ -83,7 +70,9 @@ void TestBlackboxQt::autoQrc()
 void TestBlackboxQt::cachedQml()
 {
     QDir::setCurrent(testDataDir + "/cached-qml");
-    QCOMPARE(runQbs(), 0);
+    if ((runQbs() != 0) && m_qbsStderr.contains("Dependency 'Qt.qml' not found for product 'app'"))
+        QSKIP("Qt version too old");
+
     QString dataDir = relativeBuildDir() + "/install-root/data";
     QVERIFY2(m_qbsStdout.contains("qmlcachegen must work: true")
              || m_qbsStdout.contains("qmlcachegen must work: false"),
@@ -118,7 +107,7 @@ void TestBlackboxQt::combinedMoc()
 void TestBlackboxQt::createProject()
 {
     QDir::setCurrent(testDataDir + "/create-project");
-    QVERIFY(QFile::copy(SRCDIR "/../../../examples/helloworld-qt/main.cpp",
+    QVERIFY(QFile::copy(testSourceDir + "/../../../../examples/helloworld-qt/main.cpp",
                         QDir::currentPath() + "/main.cpp"));
     QbsRunParameters createParams("create-project");
     createParams.profile.clear();
@@ -145,6 +134,11 @@ void TestBlackboxQt::dbusInterfaces()
 void TestBlackboxQt::forcedMoc()
 {
     QDir::setCurrent(testDataDir + "/forced-moc");
+    QCOMPARE(runQbs(QbsRunParameters("resolve")), 0);
+    if (m_qbsStdout.contains("using qt4"))
+        QSKIP("Qt version too old");
+    if (m_qbsStdout.contains("targetPlatform differs from hostPlatform"))
+        QSKIP("Cannot run binaries in cross-compiled build");
     QCOMPARE(runQbs(QbsRunParameters("run")), 0);
     QVERIFY2(m_qbsStderr.contains("Hello from slot"), m_qbsStderr.constData());
 }
@@ -153,6 +147,8 @@ void TestBlackboxQt::includedMocCpp()
 {
     QDir::setCurrent(testDataDir + "/included-moc-cpp");
     QCOMPARE(runQbs(), 0);
+    if (m_qbsStdout.contains("using qt4"))
+        QSKIP("Qt version too old");
     QVERIFY2(!m_qbsStdout.contains("compiling moc_myobject.cpp"), m_qbsStdout.constData());
     WAIT_FOR_NEW_TIMESTAMP();
     REPLACE_IN_FILE("myobject.cpp", "#include <moc_myobject.cpp", "// #include <moc_myobject.cpp");
@@ -204,12 +200,48 @@ void TestBlackboxQt::lrelease()
     QVERIFY(!regularFileExists(relativeProductBuildDir("lrelease-test") + "/hu.qm"));
 }
 
+void TestBlackboxQt::metaTypes_data()
+{
+    QTest::addColumn<bool>("generate");
+    QTest::addColumn<QString>("installDir");
+    QTest::newRow("don't generate") << false << QString();
+    QTest::newRow("don't generate with install info") << false << QString("blubb");
+    QTest::newRow("generate only") << true << QString();
+    QTest::newRow("generate and install") << true << QString("blubb");
+}
+
+void TestBlackboxQt::metaTypes()
+{
+    QDir::setCurrent(testDataDir + "/metatypes");
+    QFETCH(bool, generate);
+    QFETCH(QString, installDir);
+    const QStringList args{"modules.Qt.core.generateMetaTypesFile:"
+                               + QString(generate ? "true" : "false"),
+                           "modules.Qt.core.metaTypesInstallDir:" + installDir,
+                           "-v", "--force-probe-execution"};
+    QCOMPARE(runQbs(QbsRunParameters("resolve", args)), 0);
+    const bool canGenerate = m_qbsStdout.contains("can generate");
+    const bool cannotGenerate = m_qbsStdout.contains("cannot generate");
+    QVERIFY(canGenerate != cannotGenerate);
+    const bool expectFiles = generate && canGenerate;
+    const bool expectInstalledFiles = expectFiles && !installDir.isEmpty();
+    QCOMPARE(runQbs(QStringList("--clean-install-root")), 0);
+    const QString productDir = relativeProductBuildDir("mylib");
+    const QString outputDir =  productDir + "/qt.headers";
+    QVERIFY(!regularFileExists(outputDir + "/moc_unmocableclass.cpp.json"));
+    QCOMPARE(regularFileExists(outputDir + "/moc_mocableclass1.cpp.json"), expectFiles);
+    QCOMPARE(regularFileExists(outputDir + "/mocableclass2.moc.json"), expectFiles);
+    QCOMPARE(regularFileExists(productDir + "/mylib_metatypes.json"), expectFiles);
+    QCOMPARE(regularFileExists(relativeBuildDir() + "/install-root/some-prefix/" + installDir
+                               + "/mylib_metatypes.json"), expectInstalledFiles);
+}
+
 void TestBlackboxQt::mixedBuildVariants()
 {
     QDir::setCurrent(testDataDir + "/mixed-build-variants");
     const SettingsPtr s = settings();
     Profile profile(profileName(), s.get());
-    if (profile.value("qbs.toolchain").toStringList().contains("msvc")) {
+    if (profileToolchain(profile).contains("msvc")) {
         QbsRunParameters params;
         params.arguments << "qbs.buildVariant:debug";
         params.expectFailure = true;
@@ -240,6 +272,12 @@ void TestBlackboxQt::mocFlags()
     QVERIFY(runQbs(params) != 0);
 }
 
+void TestBlackboxQt::mocCompilerDefines()
+{
+    QDir::setCurrent(testDataDir + "/moc-compiler-defines");
+    QCOMPARE(runQbs(), 0);
+}
+
 void TestBlackboxQt::mocSameFileName()
 {
     QDir::setCurrent(testDataDir + "/moc-same-file-name");
@@ -247,19 +285,124 @@ void TestBlackboxQt::mocSameFileName()
     QCOMPARE(m_qbsStdout.count("compiling moc_someclass.cpp"), 2);
 }
 
+void TestBlackboxQt::noRelinkOnQDebug()
+{
+    QFETCH(QString, checkMode);
+    QFETCH(bool, expectRelink);
+
+    QVERIFY(QDir::setCurrent(testDataDir + "/no-relink-on-qdebug"));
+    rmDirR("default");
+
+    // Target check.
+    QCOMPARE(runQbs(QbsRunParameters("resolve")), 0);
+    QVERIFY2(m_qbsStdout.contains("is GCC: "), m_qbsStdout.constData());
+    QVERIFY2(m_qbsStdout.contains("is MinGW: "), m_qbsStdout.constData());
+    QVERIFY2(m_qbsStdout.contains("is Darwin: "), m_qbsStdout.constData());
+    const bool isGCCLike = m_qbsStdout.contains("is GCC: true");
+    const bool isMingw = m_qbsStdout.contains("is MinGW: true");
+    const bool isDarwin = m_qbsStdout.contains("is Darwin: true");
+    if (!isGCCLike)
+        expectRelink = false;
+    else if (isMingw || isDarwin)
+        expectRelink = true;
+
+    // Initial build.
+    QbsRunParameters params("resolve");
+    if (isGCCLike && !checkMode.isEmpty())
+        params.arguments << ("modules.cpp.exportedSymbolsCheckMode:" + checkMode);
+    QCOMPARE(runQbs(params), 0);
+    QCOMPARE(runQbs(), 0);
+    QCOMPARE(m_qbsStdout.count("linking"), 2);
+
+    // Inserting the qDebug() statement will pull in weak symbols.
+    WAIT_FOR_NEW_TIMESTAMP();
+    REPLACE_IN_FILE("lib.cpp", "// qDebug", "qDebug");
+    QCOMPARE(runQbs(), 0);
+    QCOMPARE(m_qbsStdout.count("linking"), expectRelink ? 2 : 1);
+
+    // Also check the opposite case.
+    WAIT_FOR_NEW_TIMESTAMP();
+    REPLACE_IN_FILE("lib.cpp", "qDebug", "// qDebug");
+    QCOMPARE(runQbs(), 0);
+    QCOMPARE(m_qbsStdout.count("linking"), expectRelink ? 2 : 1);
+}
+
+void TestBlackboxQt::noRelinkOnQDebug_data()
+{
+    QTest::addColumn<QString>("checkMode");
+    QTest::addColumn<bool>("expectRelink");
+    QTest::newRow("default") << QString() << false;
+    QTest::newRow("relaxed") << QString("ignore-undefined") << false;
+    QTest::newRow("strict") << QString("strict") << true;
+}
+
 void TestBlackboxQt::pkgconfig()
 {
     QDir::setCurrent(testDataDir + "/pkgconfig");
+    QCOMPARE(runQbs(QbsRunParameters("resolve")), 0);
+    if (m_qbsStdout.contains("targetPlatform differs from hostPlatform"))
+        QSKIP("Cannot run binaries in cross-compiled build");
     QbsRunParameters params;
     params.command = "run";
     QCOMPARE(runQbs(params), 0);
-    if (m_qbsStdout.contains("Skip this test"))
+    if (m_qbsStdout.contains("targetPlatform differs from hostPlatform"))
         QSKIP("pkgconfig or Qt not found");
 }
 
+void TestBlackboxQt::pkgconfigQt()
+{
+    QFETCH(QStringList, arguments);
+    QFETCH(bool, success);
+
+    QDir::setCurrent(testDataDir + "/pkgconfig-qt");
+    rmDirR(relativeBuildDir());
+
+    QbsRunParameters dumpParams("resolve", {"-f", "dump-libpath.qbs"});
+    QCOMPARE(runQbs(dumpParams), 0);
+    auto lines = QString::fromUtf8(m_qbsStdout).split('\n');
+    const QString needle = "libPath=";
+    qbs::Internal::removeIf(
+            lines, [&needle](const auto &line) { return !line.startsWith(needle); });
+    QCOMPARE(lines.size(), 1);
+    const auto libPath = lines[0].mid(needle.size());
+    auto prefix = QFileInfo(libPath).path();
+    if (prefix.endsWith("/lib") && !prefix.startsWith("/lib"))
+       prefix = QFileInfo(prefix).path();
+    const auto pkgConfigPath = libPath + "/pkgconfig/";
+    if (!QFileInfo(pkgConfigPath).exists())
+        QSKIP("No *.pc files found");
+
+    rmDirR(relativeBuildDir());
+    QbsRunParameters params("build", {"-f", "pkgconfig-qt.qbs"});
+    // need to override prefix for the downloaded Qt
+    params.environment.insert("PKG_CONFIG_QT5CORE_PREFIX", prefix);
+    params.arguments << "moduleProviders.qbspkgconfig.extraPaths:" + pkgConfigPath;
+    params.arguments << arguments;
+
+    QCOMPARE(runQbs(params) == 0, success);
+
+    if (!success)
+        QVERIFY(m_qbsStderr.contains("Dependency 'Qt.core' not found for product 'p'"));
+}
+
+void TestBlackboxQt::pkgconfigQt_data()
+{
+    QTest::addColumn<QStringList>("arguments");
+    QTest::addColumn<bool>("success");
+    QTest::newRow("pkgconfig") << QStringList() << true;
+    QTest::newRow("dummy")
+            << QStringList({"products.p.qbsModuleProviders:dummyProvider"}) << false;
+}
+
 void TestBlackboxQt::pluginMetaData()
 {
     QDir::setCurrent(testDataDir + "/plugin-meta-data");
+    QCOMPARE(runQbs(QbsRunParameters("resolve")), 0);
+    if (m_qbsStdout.contains("using qt4"))
+        QSKIP("Qt version too old");
+    if (m_qbsStdout.contains("targetPlatform differs from hostPlatform"))
+        QSKIP("Cannot run binaries in cross-compiled build");
+
     QVERIFY2(runQbs(QbsRunParameters("run", QStringList{"-p", "app"})) == 0,
              m_qbsStderr.constData());
     QVERIFY2(m_qbsStderr.contains("all ok!"), m_qbsStderr.constData());
@@ -286,7 +429,11 @@ void TestBlackboxQt::pluginSupport()
         resolveParams.arguments << "modules.m1.useDummy:true";
         resolveParams.expectFailure = true;
     }
-    QCOMPARE(runQbs(resolveParams) == 0, !invalidPlugin);
+    bool resolveResult = runQbs(resolveParams) == 0;
+    if (m_qbsStdout.contains("using qt4"))
+        QSKIP("Qt version too old");
+    QCOMPARE(resolveResult, !invalidPlugin);
+
     if (invalidPlugin) {
         QVERIFY2(m_qbsStderr.contains("Plugin 'dummy' of type 'imageformats' was requested, "
                                       "but is not available"), m_qbsStderr.constData());
@@ -326,7 +473,7 @@ void TestBlackboxQt::qmlDebugging()
     QCOMPARE(runQbs(), 0);
     const SettingsPtr s = settings();
     Profile profile(profileName(), s.get());
-    if (!profile.value("qbs.toolchain").toStringList().contains("gcc"))
+    if (!profileToolchain(profile).contains("gcc"))
         return;
     QProcess nm;
     nm.start("nm", QStringList(relativeExecutableFilePath("debuggable-app")));
@@ -347,6 +494,42 @@ void TestBlackboxQt::qobjectInObjectiveCpp()
     QCOMPARE(runQbs(), 0);
 }
 
+void TestBlackboxQt::qmlTypeRegistrar_data()
+{
+    QTest::addColumn<QString>("importName");
+    QTest::addColumn<QString>("installDir");
+    QTest::newRow("don't generate") << QString() << QString();
+    QTest::newRow("don't generate with install info") << QString() << QString("blubb");
+    QTest::newRow("generate only") << QString("People") << QString();
+    QTest::newRow("generate and install") << QString("People") << QString("blubb");
+}
+
+void TestBlackboxQt::qmlTypeRegistrar()
+{
+    QDir::setCurrent(testDataDir + "/qmltyperegistrar");
+    QFETCH(QString, importName);
+    QFETCH(QString, installDir);
+    rmDirR(relativeBuildDir());
+    const QStringList args{"modules.Qt.qml.importName:" + importName,
+                           "modules.Qt.qml.typesInstallDir:" + installDir};
+    if ((runQbs(QbsRunParameters("resolve", args)) != 0) &&
+            m_qbsStderr.contains("Dependency 'Qt.qml' not found for product 'myapp'"))
+        QSKIP("Qt version too old");
+    const bool hasRegistrar = m_qbsStdout.contains("has registrar");
+    const bool doesNotHaveRegistrar = m_qbsStdout.contains("does not have registrar");
+    QVERIFY(hasRegistrar != doesNotHaveRegistrar);
+    if (doesNotHaveRegistrar)
+        QSKIP("Qt version too old");
+    QCOMPARE(runQbs(), 0);
+    const bool enabled = !importName.isEmpty();
+    QCOMPARE(m_qbsStdout.contains("running qmltyperegistrar"), enabled);
+    QCOMPARE(m_qbsStdout.contains("compiling myapp_qmltyperegistrations.cpp"), enabled);
+    const QString buildDir = relativeProductBuildDir("myapp");
+    QCOMPARE(regularFileExists(buildDir + "/myapp.qmltypes"), enabled);
+    QCOMPARE(regularFileExists(relativeBuildDir() + "/install-root/" + installDir
+                               + "/myapp.qmltypes"), enabled && !installDir.isEmpty());
+}
+
 void TestBlackboxQt::qtKeywords()
 {
     QDir::setCurrent(testDataDir + "/qt-keywords");
@@ -364,13 +547,25 @@ void TestBlackboxQt::qtKeywords()
 void TestBlackboxQt::quickCompiler()
 {
     QDir::setCurrent(testDataDir + "/quick-compiler");
-    QCOMPARE(runQbs(), 0);
+    if ((runQbs() != 0) &&
+            m_qbsStderr.contains("'Qt.quick' has version 4.8.7, but it needs to be at least 5.0.0."))
+        QSKIP("Qt version too old");
     const bool hasCompiler = m_qbsStdout.contains("compiler available");
     const bool doesNotHaveCompiler = m_qbsStdout.contains("compiler not available");
     QVERIFY2(hasCompiler || doesNotHaveCompiler, m_qbsStdout.constData());
     QCOMPARE(m_qbsStdout.contains("compiling qml_subdir_test_qml.cpp"), hasCompiler);
     if (doesNotHaveCompiler)
         QSKIP("qtquickcompiler not available");
+    QVERIFY2(m_qbsStdout.contains("generating loader source"), m_qbsStdout.constData());
+
+    QCOMPARE(runQbs(), 0);
+    QVERIFY2(!m_qbsStdout.contains("generating loader source"), m_qbsStdout.constData());
+
+    WAIT_FOR_NEW_TIMESTAMP();
+    touch("qml/subdir/test.qml");
+    QCOMPARE(runQbs(), 0);
+    QVERIFY2(m_qbsStdout.contains("generating loader source"), m_qbsStdout.constData());
+
     QCOMPARE(runQbs(QbsRunParameters(QStringList{"config:off",
                                                  "modules.Qt.quick.useCompiler:false"})), 0);
     QVERIFY2(m_qbsStdout.contains("compiling"), m_qbsStdout.constData());
@@ -380,6 +575,9 @@ void TestBlackboxQt::quickCompiler()
 void TestBlackboxQt::qtScxml()
 {
     QDir::setCurrent(testDataDir + "/qtscxml");
+    QCOMPARE(runQbs(QbsRunParameters("resolve")), 0);
+    if (m_qbsStdout.contains("targetPlatform differs from hostPlatform"))
+        QSKIP("Cannot run binaries in cross-compiled build");
     QCOMPARE(runQbs(), 0);
     if (m_qbsStdout.contains("QtScxml not present"))
         QSKIP("QtScxml module not present");
@@ -459,6 +657,11 @@ void TestBlackboxQt::track_qobject_change()
 void TestBlackboxQt::track_qrc()
 {
     QDir::setCurrent(testDataDir + "/qrc");
+    QCOMPARE(runQbs(QbsRunParameters("resolve")), 0);
+    if (m_qbsStdout.contains("using qt4"))
+        QSKIP("Qt version too old");
+    if (m_qbsStdout.contains("targetPlatform differs from hostPlatform"))
+        QSKIP("Cannot run binaries in cross-compiled build");
     QCOMPARE(runQbs(QbsRunParameters("run")), 0);
     QVERIFY2(m_qbsStdout.contains("rcc"), m_qbsStdout.constData());
     QVERIFY2(!m_qbsStdout.contains("compiling test.cpp"), m_qbsStdout.constData());
